"use strict";
var logger_1 = require("./logger");
var tsquery_1 = require("@phenomnomnominal/tsquery");
var typescript_1 = require("typescript");
function findConstructorsWithParameters(fileContent, log) {
    var result = [];
    var ast = tsquery_1.tsquery.ast(fileContent);
    var classes = tsquery_1.tsquery(ast, "ClassDeclaration");
    for (var _i = 0, classes_1 = classes; _i < classes_1.length; _i++) {
        var classNode = classes_1[_i];
        var annotatableClass = {};
        if (classNode.kind !== typescript_1.SyntaxKind.ClassDeclaration)
            continue;
        annotatableClass.classDeclaration = classNode;
        if (!annotatableClass.classDeclaration.name)
            continue;
        log.debug("Class: " + annotatableClass.classDeclaration.name.getText());
        for (var _a = 0, _b = annotatableClass.classDeclaration.members; _a < _b.length; _a++) {
            var member = _b[_a];
            if (member.kind !== typescript_1.SyntaxKind.Constructor)
                continue;
            annotatableClass.constructorDeclaration = member;
            annotatableClass.parameterDeclarations = annotatableClass.constructorDeclaration.parameters.map(function (param) { return param; });
            if (annotatableClass.parameterDeclarations.length > 0) {
                result.push(annotatableClass);
                log.debug("\tctor args: " + annotatableClass.parameterDeclarations.map(function (p) { return p.name.getText(); }));
            }
            else {
                log.debug("\tctor without args");
            }
        }
    }
    return result;
}
function annotate(fileContent, classToAnnotate, log) {
    var ctorParamsList = classToAnnotate.parameterDeclarations.map(function (p) { return "\"" + p.name.getText() + "\""; });
    var injectStatement = "\nstatic $inject = [" + ctorParamsList.join(",") + "];\n";
    log.debug("Inject statement: " + injectStatement);
    var newFileContent = insertAt(fileContent, classToAnnotate.constructorDeclaration.pos, injectStatement);
    return newFileContent;
}
function insertAt(source, index, str) {
    return source.slice(0, index) + str + source.slice(index);
}
/**
 * Accepts content of typescript file and returns it with added $inject annotations in each class.
 * @param tsFileContent
 * @param log
 */
function annotateTs(tsFileContent, log) {
    if (log === void 0) { log = logger_1.noLog; }
    var annotatableClasses = findConstructorsWithParameters(tsFileContent, log);
    for (var i = annotatableClasses.length - 1; i >= 0; i--) {
        var annotatableClass = annotatableClasses[i];
        tsFileContent = annotate(tsFileContent, annotatableClass, log);
    }
    return tsFileContent;
}
module.exports = annotateTs;
//# sourceMappingURL=index.js.map